#include "alt_types.h"
#include "alt_2_wire.h"
#include "alt_eeprom.h"

static struct device devices[] = {
    { 0,    0x10,   0xA0,   0xA1    },
    { 8,    0x400,  0xA0,   0xA1    },
    { 16,   0x800,  0xA0,   0xA1    },
};

static struct device *device = &(devices[0]);

static error_t get_board_mac_addr(unsigned char mac_addr[6],struct device *device)
{
  error_t error = 0;

  alt_u32 signature = 0;

  alt_two_wire s_eeprom;
  alt_two_wire* eeprom = &s_eeprom;

/* Initialize the EEPROM. */
  
  eeprom->scl_pio = PIO_ID_EEPROM_SCL_BASE;
  eeprom->sda_pio = PIO_ID_EEPROM_DAT_BASE;

  alt_2_wireInit(eeprom);
  alt_2_wireSetDelay(10);

  printf("EEPROM device 24LC%d size is %x\n", device->id, device->size);

  char buf[10] = { 0 };

  if (!error)
  {
    if(!readRandom(device, 0, buf, 10)) {
      printf( "Successfully read 16 bytes from EEPROM.\n");
      signature = (buf[2]*0x10000 & 0xff0000) + (buf[3]*0x100 & 0xff00) + (buf[4] & 0xff);
    }
    printf( "Signature = 0x%x.\n", (unsigned int) signature);
    if (signature != 0x0007ed)
    {
      printf( "Your EEK LCD daughtercard is not programmed with a correct MAC Address!\n");
      printf( "Please contact Altera support for instructions on how to correct this problem!\n");
    }
  }
  
  if (!error)
  {
    mac_addr[0] = buf[2] & 0xFF;
    mac_addr[1] = buf[3] & 0xFF;
    mac_addr[2] = buf[4] & 0xFF;
    mac_addr[3] = buf[5] & 0xFF;
    mac_addr[4] = buf[6] & 0xFF;
    mac_addr[5] = buf[7] & 0xFF;
    
    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n", 
            mac_addr[0],
            mac_addr[1],
            mac_addr[2],
            mac_addr[3],
            mac_addr[4],
            mac_addr[5]);

  }
  
  return error;
}


/*
* get_mac_addr
*
* Read the MAC address in a board specific way
*
*/
int neek_get_mac_addr(unsigned char mac_addr[6])
{
    return (get_board_mac_addr(mac_addr, device));
}

/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2008 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
