#include "alt_types.h"
#include <sys/alt_cache.h>
#include "alt_video_display.h"
#include "app_selector.h"

#define LCD_WIDTH		(800)
#define LCD_HEIGHT		(480)

#define GFX_CHAR_HEIGHT		(12)
#define GFX_CHAR_WIDTH		(8)
#define LCD_MULTIPLIER		(2)
#define LCD_CHAR_HEIGHT		(GFX_CHAR_HEIGHT * LCD_MULTIPLIER)
#define LCD_CHAR_WIDTH		(GFX_CHAR_WIDTH * LCD_MULTIPLIER)
#define GFX_CHAR_COLS		(LCD_WIDTH / LCD_CHAR_WIDTH)
#define GFX_CHAR_ROWS		(LCD_HEIGHT / LCD_CHAR_HEIGHT)

static struct {
	alt_u32 *frameBuffer;
}GFX_GLOBALS;

static int gfx_current_column = 0;

extern unsigned char character_bitmaps[128][12];

void init_gfx_display(void) {

	// This is our LCD display
	alt_video_display* display;
	  
	// This is our LCD configuration port
	alt_tpo_lcd lcd_serial;
	
	// Initialize and start the LCD display
	display = alt_video_display_init(	ALT_VIDEO_DISPLAY_SGDMA_NAME,         // char* sgdma_name
										ALT_VIDEO_DISPLAY_COLS,               // int width
										ALT_VIDEO_DISPLAY_ROWS,               // int height
										ALT_VIDEO_DISPLAY_COLOR_DEPTH,        // int color_depth
										ALT_VIDEO_DISPLAY_USE_HEAP,           // int buffer_location
										ALT_VIDEO_DISPLAY_USE_HEAP,           // int descriptor_location
										1 );                                  // int num_buffers
	
	// Set the Gamma Curve and Resolution of the LCD
	AsInitTpoLcd( &lcd_serial );

	// get the frame buffer pointer and store it in our GFX_GLOBALS
	GFX_GLOBALS.frameBuffer = (alt_32*)((alt_u32)(display->buffer_ptrs[0]->buffer) & 0x7FFFFFFF);

	// initialize the current column global
	gfx_current_column = 0;
}

static void scroll_gfx_buffer_up_one_char_row(void) {
	int i;

	alt_u32* dstFrameBuffer = GFX_GLOBALS.frameBuffer;
	alt_u32* srcFrameBuffer = dstFrameBuffer + (LCD_WIDTH * LCD_CHAR_HEIGHT);
	
	for(i = 0 ; i < (LCD_HEIGHT - LCD_CHAR_HEIGHT) * LCD_WIDTH ; i++) {
		*dstFrameBuffer++ = *srcFrameBuffer++;
	}
}

static void clear_gfx_buffer_last_char_row(void) {
	int i;
	alt_u32* frameBuffer = GFX_GLOBALS.frameBuffer;
	
	frameBuffer += ((LCD_HEIGHT - LCD_CHAR_HEIGHT) * LCD_WIDTH);
	
	for(i = 0 ; i < (LCD_WIDTH * LCD_CHAR_HEIGHT) ; i++) {
		*frameBuffer++ = 0x00000000;		
	}
}

static void gfx_display_put_char(char c) {
	int i, x, y;
	alt_u8 temp;
	alt_u32 temp_0, temp_1, temp_2, temp_3, temp_4, temp_5, temp_6, temp_7;
	alt_u32* frameBuffer = GFX_GLOBALS.frameBuffer;
	
	for(i = 0 ; i < GFX_CHAR_HEIGHT ; i++) {
		temp = character_bitmaps[(int)(c)][i];
		temp_0 = (temp & 0x80) ? (0xFFFFFFFF) : (0x00000000);
		temp_1 = (temp & 0x40) ? (0xFFFFFFFF) : (0x00000000);
		temp_2 = (temp & 0x20) ? (0xFFFFFFFF) : (0x00000000);
		temp_3 = (temp & 0x10) ? (0xFFFFFFFF) : (0x00000000);
		temp_4 = (temp & 0x08) ? (0xFFFFFFFF) : (0x00000000);
		temp_5 = (temp & 0x04) ? (0xFFFFFFFF) : (0x00000000);
		temp_6 = (temp & 0x02) ? (0xFFFFFFFF) : (0x00000000);
		temp_7 = (temp & 0x01) ? (0xFFFFFFFF) : (0x00000000);
		
		for(y = 0 ; y < LCD_MULTIPLIER ; y++) {
			for(x = 0 ; x < LCD_MULTIPLIER ; x++) {
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 0)] = temp_0;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 1)] = temp_1;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 2)] = temp_2;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 3)] = temp_3;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 4)] = temp_4;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 5)] = temp_5;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 6)] = temp_6;
				frameBuffer[(((LCD_HEIGHT - LCD_CHAR_HEIGHT) + y + (i * LCD_MULTIPLIER)) * LCD_WIDTH) + (gfx_current_column * LCD_CHAR_WIDTH) + x + (LCD_MULTIPLIER * 7)] = temp_7;
			}
		}
	}
}

void gfx_display_put_str(char *str) {
	char c;
	
	c = *str++;
	while(c != '\0') {
		if(c == '\n') {
			scroll_gfx_buffer_up_one_char_row();
			clear_gfx_buffer_last_char_row();
			gfx_current_column = 0;
		} else if(c == '\r') {
			clear_gfx_buffer_last_char_row();
			gfx_current_column = 0;
		} else if(c == '\b') {
			gfx_current_column--;
			if(gfx_current_column < 0){
				gfx_current_column = 0;					
			}
			gfx_display_put_char(' ');
		} else if(gfx_current_column < GFX_CHAR_COLS) {
			gfx_display_put_char(c);
			gfx_current_column++;
		}
		c = *str++;
	}
	
	alt_dcache_flush_all();
}

/*****************************************************************************
 *  Function: AsInitTpoLcd
 *
 *  Purpose: Initializes the gamma and resolution of the LCD screen
 * 
 *  Returns: 0
 ****************************************************************************/
int AsInitTpoLcd( alt_tpo_lcd* lcd_serial )
{

  // Specify base addresses of the each communication bus PIO,
  // as defined in system.h
  lcd_serial->scen_pio = LCD_I2C_EN_BASE;
  lcd_serial->scl_pio  = LCD_I2C_SCL_BASE;
  lcd_serial->sda_pio  = LCD_I2C_SDAT_BASE;

  // Write the default gamma curve, use 800x480 resolution (default),
  // and verify that the LCD is alive by reading back its Chip/ID
  // register. Most users will only need to call this routine, and
  // verify that it completed successfully.
  alt_tpo_lcd_init( lcd_serial, ALT_VIDEO_DISPLAY_COLS, ALT_VIDEO_DISPLAY_ROWS );

  return( 0 );
}
